@@ -1,8 +1,13 @@
 #pragma once
+#include "../ds/concept.h"
 #include "../ds/defines.h"
+#include "../ds/ptrwrap.h"
+#include "aal_concept.h"
+#include "aal_consts.h"
 
 #include <chrono>
 #include <cstdint>
+#include <utility>
 
 #if defined(__i386__) || defined(_M_IX86) || defined(_X86_) || \
   defined(__amd64__) || defined(__x86_64__) || defined(_M_X64) || \
@@ -29,38 +34,6 @@
 
 namespace snmalloc
 {
-  /**
-   * Flags in a bitfield of attributes of this architecture, much like
-   * PalFeatures.
-   */
-  enum AalFeatures : uint64_t
-  {
-    /**
-     * This architecture does not discriminate between integers and pointers,
-     * and so may use bit operations on pointer values.
-     */
-    IntegerPointers = (1 << 0),
-    /**
-     * This architecture cannot access cpu cycles counters.
-     */
-    NoCpuCycleCounters = (1 << 1),
-    /**
-     * This architecture enforces strict pointer provenance; we bound the
-     * pointers given out on malloc() and friends and must, therefore retain
-     * internal high-privilege pointers for recycling memory on free().
-     */
-    StrictProvenance = (1 << 2),
-  };
-
-  enum AalName : int
-  {
-    ARM,
-    PowerPC,
-    X86,
-    X86_SGX,
-    Sparc,
-  };
-
   /**
    * Architecture Abstraction Layer. Includes default implementations of some
    * functions using compiler builtins.  Falls back to the definitions in the
@@ -102,7 +75,7 @@ namespace snmalloc
      * architectures to avoid needing to implement a custom `prefetch` method
      * if they are used only with a compiler that provides the builtin.
      */
-    static inline void prefetch(void* ptr)
+    static inline void prefetch(void* ptr) noexcept
     {
 #if __has_builtin(__builtin_prefetch) && !defined(SNMALLOC_NO_AAL_BUILTINS)
       __builtin_prefetch(ptr);
@@ -119,7 +92,7 @@ namespace snmalloc
      * architectures to avoid needing to implement a custom `tick` method
      * if they are used only with a compiler that provides the builtin.
      */
-    static inline uint64_t tick()
+    static inline uint64_t tick() noexcept
     {
       if constexpr (
         (Arch::aal_features & NoCpuCycleCounters) == NoCpuCycleCounters)
@@ -142,6 +115,48 @@ namespace snmalloc
     }
   };
 
+  template<class Arch>
+  class AAL_NoStrictProvenance : public Arch
+  {
+    static_assert(
+      (Arch::aal_features & StrictProvenance) == 0,
+      "AAL_NoStrictProvenance requires what it says on the tin");
+
+  public:
+    /**
+     * For architectures which do not enforce StrictProvenance, we can just
+     * perform an underhanded bit of type-casting.
+     */
+    template<
+      typename T,
+      enum capptr_bounds nbounds,
+      enum capptr_bounds obounds,
+      typename U = T>
+    static SNMALLOC_FAST_PATH CapPtr<T, nbounds>
+    capptr_bound(CapPtr<U, obounds> a, size_t size) noexcept
+    {
+      // Impose constraints on bounds annotations.
+      static_assert(
+        obounds == CBArena || obounds == CBChunkD || obounds == CBChunk ||
+        obounds == CBChunkE);
+      static_assert(capptr_is_bounds_refinement<obounds, nbounds>());
+
+      UNUSED(size);
+      return CapPtr<T, nbounds>(a.template as_static<T>().unsafe_capptr);
+    }
+
+    /**
+     * For architectures which do not enforce StrictProvenance, there's nothing
+     * to be done, so just return the pointer unmodified with new annotation.
+     */
+    template<typename T, capptr_bounds BOut, capptr_bounds BIn>
+    static SNMALLOC_FAST_PATH CapPtr<T, BOut>
+    capptr_rebound(CapPtr<void, BOut> a, CapPtr<T, BIn> r) noexcept
+    {
+      UNUSED(a);
+      return CapPtr<T, BOut>(r.unsafe_capptr);
+    }
+  };
 } // namespace snmalloc
 
 #if defined(PLATFORM_IS_X86)
@@ -158,9 +173,9 @@ namespace snmalloc
 
 namespace snmalloc
 {
-  using Aal = AAL_Generic<AAL_Arch>;
+  using Aal = AAL_Generic<AAL_NoStrictProvenance<AAL_Arch>>;
 
-  template<AalFeatures F, typename AAL = Aal>
+  template<AalFeatures F, SNMALLOC_CONCEPT(ConceptAAL) AAL = Aal>
   constexpr static bool aal_supports = (AAL::aal_features & F) == F;
 } // namespace snmalloc
 
