@@ -20,6 +20,20 @@
 #  define SNMALLOC_COLD __attribute__((cold))
 #endif
 
+#if defined(__cpp_constinit) && __cpp_constinit >= 201907
+#  define SNMALLOC_CONSTINIT_FN constinit
+#  define SNMALLOC_CONSTINIT_STATIC constinit const
+#else
+#  define SNMALLOC_CONSTINIT_FN constexpr
+#  define SNMALLOC_CONSTINIT_STATIC constexpr
+#endif
+
+#if defined(__cpp_consteval)
+#  define SNMALLOC_CONSTEVAL consteval
+#else
+#  define SNMALLOC_CONSTEVAL constexpr
+#endif
+
 #if !defined(__clang__) && defined(__GNUC__)
 #  define GCC_NOT_CLANG
 #endif
@@ -30,6 +44,14 @@
 #  endif
 #endif
 
+#ifdef __APPLE__
+#  define SNMALLOC_FORCE_BSS __attribute__((section("__DATA,__bss")))
+#elif defined(__ELF__)
+#  define SNMALLOC_FORCE_BSS __attribute__((section(".bss")))
+#else
+#  define SNMALLOC_FORCE_BSS
+#endif
+
 #ifndef __has_builtin
 #  define __has_builtin(x) 0
 #endif
