@@ -96,6 +96,8 @@ namespace snmalloc
                       ChunkMap,
                       IsQueueInline>>
   {
+    friend RemoteCache;
+
     LargeAlloc<MemoryProvider> large_allocator;
     ChunkMap chunk_map;
     LocalEntropy entropy;
@@ -504,175 +506,17 @@ namespace snmalloc
   private:
     using alloc_id_t = typename Remote::alloc_id_t;
 
-    /*
-     * A singly-linked list of Remote objects, supporting append and
-     * take-all operations.  Intended only for the private use of this
-     * allocator; the Remote objects here will later be taken and pushed
-     * to the inter-thread message queues.
-     */
-    struct RemoteList
-    {
-      /*
-       * A stub Remote object that will always be the head of this list;
-       * never taken for further processing.
-       */
-      Remote head{};
-
-      CapPtr<Remote, CBAlloc> last{&head};
-
-      void clear()
-      {
-        last = CapPtr<Remote, CBAlloc>(&head);
-      }
-
-      bool empty()
-      {
-        return address_cast(last) == address_cast(&head);
-      }
-    };
-
-    struct RemoteCache
-    {
-      /**
-       * The total amount of memory we are waiting for before we will dispatch
-       * to other allocators. Zero or negative mean we should dispatch on the
-       * next remote deallocation. This is initialised to the 0 so that we
-       * always hit a slow path to start with, when we hit the slow path and
-       * need to dispatch everything, we can check if we are a real allocator
-       * and lazily provide a real allocator.
-       */
-      int64_t capacity{0};
-      std::array<RemoteList, REMOTE_SLOTS> list{};
-
-      /// Used to find the index into the array of queues for remote
-      /// deallocation
-      /// r is used for which round of sending this is.
-      inline size_t get_slot(size_t id, size_t r)
-      {
-        constexpr size_t allocator_size = sizeof(Allocator<
-                                                 NeedsInitialisation,
-                                                 InitThreadAllocator,
-                                                 MemoryProvider,
-                                                 ChunkMap,
-                                                 IsQueueInline>);
-        constexpr size_t initial_shift =
-          bits::next_pow2_bits_const(allocator_size);
-        static_assert(
-          initial_shift >= 8,
-          "Can't embed sizeclass_t into allocator ID low bits");
-        SNMALLOC_ASSERT((initial_shift + (r * REMOTE_SLOT_BITS)) < 64);
-        return (id >> (initial_shift + (r * REMOTE_SLOT_BITS))) & REMOTE_MASK;
-      }
-
-      SNMALLOC_FAST_PATH void dealloc(
-        alloc_id_t target_id,
-        CapPtr<FreeObject, CBAlloc> p,
-        sizeclass_t sizeclass)
-      {
-        this->capacity -= sizeclass_to_size(sizeclass);
-        auto r = p.template as_reinterpret<Remote>();
-
-        r->set_info(target_id, sizeclass);
-
-        RemoteList* l = &list[get_slot(target_id, 0)];
-        l->last->non_atomic_next = r;
-        l->last = r;
-      }
-
-      void post(LargeAlloc<MemoryProvider>* large_allocator, alloc_id_t id)
-      {
-        // When the cache gets big, post lists to their target allocators.
-        capacity = REMOTE_CACHE;
-
-        size_t post_round = 0;
-
-        while (true)
-        {
-          auto my_slot = get_slot(id, post_round);
-
-          for (size_t i = 0; i < REMOTE_SLOTS; i++)
-          {
-            if (i == my_slot)
-              continue;
-
-            RemoteList* l = &list[i];
-            CapPtr<Remote, CBAlloc> first = l->head.non_atomic_next;
-
-            if (!l->empty())
-            {
-              // Send all slots to the target at the head of the list.
-              auto first_auth =
-                large_allocator->template capptr_amplify<Remote>(first);
-              auto super = Superslab::get(first_auth);
-              super->get_allocator()->message_queue.enqueue(first, l->last);
-              l->clear();
-            }
-          }
-
-          RemoteList* resend = &list[my_slot];
-          if (resend->empty())
-            break;
-
-          // Entries could map back onto the "resend" list,
-          // so take copy of the head, mark the last element,
-          // and clear the original list.
-          CapPtr<Remote, CBAlloc> r = resend->head.non_atomic_next;
-          resend->last->non_atomic_next = nullptr;
-          resend->clear();
-
-          post_round++;
-
-          while (r != nullptr)
-          {
-            // Use the next N bits to spread out remote deallocs in our own
-            // slot.
-            size_t slot = get_slot(r->trunc_target_id(), post_round);
-            RemoteList* l = &list[slot];
-            l->last->non_atomic_next = r;
-            l->last = r;
-
-            r = r->non_atomic_next;
-          }
-        }
-      }
-    };
-
     SlabList small_classes[NUM_SMALL_CLASSES];
     DLList<Mediumslab, CapPtrCBChunkE> medium_classes[NUM_MEDIUM_CLASSES];
 
     DLList<Superslab, CapPtrCBChunk> super_available;
     DLList<Superslab, CapPtrCBChunk> super_only_short_available;
 
-    RemoteCache remote;
+    RemoteCache remote_cache;
 
     std::conditional_t<IsQueueInline, RemoteAllocator, RemoteAllocator*>
       remote_alloc;
 
-#ifdef CACHE_FRIENDLY_OFFSET
-    size_t remote_offset = 0;
-
-    template<capptr_bounds B>
-    CapPtr<FreeObject, B>
-    apply_cache_friendly_offset(CapPtr<void, B> p, sizeclass_t sizeclass)
-    {
-      size_t mask = sizeclass_to_cache_friendly_mask(sizeclass);
-
-      size_t offset = remote_offset & mask;
-      remote_offset += CACHE_FRIENDLY_OFFSET;
-
-      return CapPtr<FreeObject, B>(reinterpret_cast<FreeObject*>(
-        reinterpret_cast<uintptr_t>(p.unsafe_capptr) + offset));
-    }
-#else
-    template<capptr_bounds B>
-    static CapPtr<FreeObject, B>
-    apply_cache_friendly_offset(CapPtr<void, B> p, sizeclass_t sizeclass)
-    {
-      UNUSED(sizeclass);
-      return p.template as_static<FreeObject>();
-    }
-#endif
-
     auto* public_state()
     {
       if constexpr (IsQueueInline)
@@ -865,26 +709,21 @@ namespace snmalloc
 
     SNMALLOC_FAST_PATH void handle_dealloc_remote(CapPtr<Remote, CBAlloc> p)
     {
-      if (likely(p->trunc_target_id() == get_trunc_id()))
+      auto target_id = Remote::trunc_target_id(p, &large_allocator);
+      if (likely(target_id == get_trunc_id()))
       {
         // Destined for my slabs
         auto p_auth = large_allocator.template capptr_amplify<Remote>(p);
         auto super = Superslab::get(p_auth);
-
-        check_client(
-          p->trunc_target_id() == super->get_allocator()->trunc_id(),
-          "Detected memory corruption.  Potential use-after-free");
-
-        dealloc_not_large_local(super, p, p->sizeclass());
+        auto sizeclass = p->sizeclass();
+        dealloc_not_large_local(super, Remote::clear(p), sizeclass);
       }
       else
       {
         // Merely routing; despite the cast here, p is going to be cast right
         // back to a Remote.
-        remote.dealloc(
-          p->trunc_target_id(),
-          p.template as_reinterpret<FreeObject>(),
-          p->sizeclass());
+        remote_cache.dealloc<Allocator>(
+          target_id, p.template as_reinterpret<void>(), p->sizeclass());
       }
     }
 
@@ -895,9 +734,7 @@ namespace snmalloc
       {
         auto p_auth = large_allocator.capptr_amplify(p);
         auto super = Superslab::get(p_auth);
-        auto offseted = apply_cache_friendly_offset(p, sizeclass)
-                          .template as_reinterpret<Remote>();
-        dealloc_not_large_local(super, offseted, sizeclass);
+        dealloc_not_large_local(super, p, sizeclass);
       }
       else
       {
@@ -905,9 +742,11 @@ namespace snmalloc
       }
     }
 
+    // TODO: Adjust when medium slab same as super slab.
+    //    Second parameter should be a FreeObject.
     SNMALLOC_FAST_PATH void dealloc_not_large_local(
       CapPtr<Superslab, CBChunkD> super,
-      CapPtr<Remote, CBAlloc> p_offseted,
+      CapPtr<void, CBAlloc> p,
       sizeclass_t sizeclass)
     {
       // Guard against remote queues that have colliding IDs
@@ -916,18 +755,26 @@ namespace snmalloc
       if (likely(sizeclass < NUM_SMALL_CLASSES))
       {
         SNMALLOC_ASSERT(super->get_kind() == Super);
-        auto slab =
-          Metaslab::get_slab(Aal::capptr_rebound(super.as_void(), p_offseted));
-        small_dealloc_offseted(
-          super, slab, FreeObject::make(p_offseted), sizeclass);
+        check_client(
+          super->get_kind() == Super,
+          "Heap Corruption: Sizeclass of remote dealloc corrupt.");
+        auto slab = Metaslab::get_slab(Aal::capptr_rebound(super.as_void(), p));
+        check_client(
+          super->get_meta(slab)->sizeclass() == sizeclass,
+          "Heap Corruption: Sizeclass of remote dealloc corrupt.");
+        small_dealloc_offseted(super, slab, p, sizeclass);
       }
       else
       {
-        SNMALLOC_ASSERT(super->get_kind() == Medium);
-        medium_dealloc_local(
-          super.template as_reinterpret<Mediumslab>(),
-          Remote::clear(p_offseted, sizeclass),
-          sizeclass);
+        auto medium = super.template as_reinterpret<Mediumslab>();
+        SNMALLOC_ASSERT(medium->get_kind() == Medium);
+        check_client(
+          medium->get_kind() == Medium,
+          "Heap Corruption: Sizeclass of remote dealloc corrupt.");
+        check_client(
+          medium->get_sizeclass() == sizeclass,
+          "Heap Corruption: Sizeclass of remote dealloc corrupt.");
+        medium_dealloc_local(medium, p, sizeclass);
       }
     }
 
@@ -944,11 +791,11 @@ namespace snmalloc
       }
 
       // Our remote queues may be larger due to forwarding remote frees.
-      if (likely(remote.capacity > 0))
+      if (likely(remote_cache.capacity > 0))
         return;
 
       stats().remote_post();
-      remote.post(&large_allocator, get_trunc_id());
+      remote_cache.post<Allocator>(this, get_trunc_id());
     }
 
     /**
@@ -1077,14 +924,15 @@ namespace snmalloc
       {
         stats().alloc_request(size);
         stats().sizeclass_alloc(sizeclass);
-        auto p = remove_cache_friendly_offset(fl.take(entropy), sizeclass);
+        auto p = fl.take(entropy);
         if constexpr (zero_mem == YesZero)
         {
           pal_zero<typename MemoryProvider::Pal>(
             p, sizeclass_to_size(sizeclass));
         }
 
-        return capptr_export(p);
+        // TODO: Should this be zeroing the next pointer?
+        return capptr_export(p.as_void());
       }
 
       if (likely(!has_messages()))
@@ -1201,14 +1049,15 @@ namespace snmalloc
       SNMALLOC_ASSERT(ffl.empty());
       Slab::alloc_new_list(bp, ffl, rsize, entropy);
 
-      auto p = remove_cache_friendly_offset(ffl.take(entropy), sizeclass);
+      auto p = ffl.take(entropy);
 
       if constexpr (zero_mem == YesZero)
       {
         pal_zero<typename MemoryProvider::Pal>(p, sizeclass_to_size(sizeclass));
       }
 
-      return capptr_export(p);
+      // TODO: Should this be zeroing the next pointer?
+      return capptr_export(p.as_void());
     }
 
     /**
@@ -1266,7 +1115,7 @@ namespace snmalloc
       sizeclass_t sizeclass)
     {
       check_client(
-        Metaslab::is_start_of_object(Slab::get_meta(slab), address_cast(p_ret)),
+        Slab::get_meta(slab)->is_start_of_object(address_cast(p_ret)),
         "Not deallocating start of an object");
 
       small_dealloc_start(super, slab, p_auth, p_ret, sizeclass);
@@ -1289,8 +1138,7 @@ namespace snmalloc
 
       if (likely(target == public_state()))
       {
-        auto offseted = apply_cache_friendly_offset(p, sizeclass);
-        small_dealloc_offseted(super, slab, offseted, sizeclass);
+        small_dealloc_offseted(super, slab, p, sizeclass);
       }
       else
         remote_dealloc(target, p, sizeclass);
@@ -1299,12 +1147,12 @@ namespace snmalloc
     SNMALLOC_FAST_PATH void small_dealloc_offseted(
       CapPtr<Superslab, CBChunkD> super,
       CapPtr<Slab, CBChunkD> slab,
-      CapPtr<FreeObject, CBAlloc> p,
+      CapPtr<void, CBAlloc> p,
       sizeclass_t sizeclass)
     {
       stats().sizeclass_dealloc(sizeclass);
 
-      small_dealloc_offseted_inner(super, slab, p, sizeclass);
+      small_dealloc_offseted_inner(super, slab, FreeObject::make(p), sizeclass);
     }
 
     SNMALLOC_FAST_PATH void small_dealloc_offseted_inner(
@@ -1660,11 +1508,10 @@ namespace snmalloc
       // Check whether this will overflow the cache first.  If we are a fake
       // allocator, then our cache will always be full and so we will never hit
       // this path.
-      if (remote.capacity > 0)
+      if (remote_cache.capacity > 0)
       {
         stats().remote_free(sizeclass);
-        auto offseted = apply_cache_friendly_offset(p, sizeclass);
-        remote.dealloc(target->trunc_id(), offseted, sizeclass);
+        remote_cache.dealloc<Allocator>(target->trunc_id(), p, sizeclass);
         return;
       }
 
@@ -1702,11 +1549,10 @@ namespace snmalloc
       handle_message_queue();
 
       stats().remote_free(sizeclass);
-      auto offseted = apply_cache_friendly_offset(p_auth, sizeclass);
-      remote.dealloc(target->trunc_id(), offseted, sizeclass);
+      remote_cache.dealloc<Allocator>(target->trunc_id(), p_auth, sizeclass);
 
       stats().remote_post();
-      remote.post(&large_allocator, get_trunc_id());
+      remote_cache.post<Allocator>(this, get_trunc_id());
     }
 
     ChunkMap& chunkmap()
