@@ -72,7 +72,7 @@ namespace snmalloc
  *     internal tree node in the non-flat pagemap).
  */
 #  define SNMALLOC_MAX_FLATPAGEMAP_SIZE \
-    (pal_supports<LazyCommit> ? 256ULL * 1024 * 1024 : PAGEMAP_NODE_SIZE)
+    (pal_supports<LazyCommit, Pal> ? 256ULL * 1024 * 1024 : PAGEMAP_NODE_SIZE)
 #endif
   static constexpr bool CHUNKMAP_USE_FLATPAGEMAP =
     SNMALLOC_MAX_FLATPAGEMAP_SIZE >=
@@ -81,83 +81,18 @@ namespace snmalloc
   using ChunkmapPagemap = std::conditional_t<
     CHUNKMAP_USE_FLATPAGEMAP,
     FlatPagemap<SUPERSLAB_BITS, uint8_t>,
-    Pagemap<SUPERSLAB_BITS, uint8_t, 0>>;
+    Pagemap<SUPERSLAB_BITS, uint8_t, 0, DefaultPrimAlloc>>;
+
+  struct ForChunkmap
+  {};
+  using GlobalChunkmap = GlobalPagemapTemplate<ChunkmapPagemap, ForChunkmap>;
 
   /**
-   * Mixin used by `ChunkMap` to directly access the pagemap via a global
-   * variable.  This should be used from within the library or program that
-   * owns the pagemap.
-   *
-   * This class makes the global pagemap a static field so that its name
-   * includes the type mangling.  If two compilation units try to instantiate
-   * two different types of pagemap then they will see two distinct pagemaps.
-   * This will prevent allocating with one and freeing with the other (because
-   * the memory will show up as not owned by any allocator in the other
-   * compilation unit) but will prevent the same memory being interpreted as
-   * having two different types.
+   * Optionally exported function that accesses the global chunkmap pagemap
+   * provided by a shared library.
    */
-  template<typename T>
-  class GlobalPagemapTemplate
-  {
-    /**
-     * The global pagemap variable.  The name of this symbol will include the
-     * type of `T`.
-     */
-    inline static T global_pagemap;
-
-  public:
-    /**
-     * Returns the pagemap.
-     */
-    static ChunkmapPagemap& pagemap()
-    {
-      return global_pagemap;
-    }
-  };
-
-  using GlobalPagemap = GlobalPagemapTemplate<ChunkmapPagemap>;
-
-  /**
-   * Optionally exported function that accesses the global pagemap provided by
-   * a shared library.
-   */
-  extern "C" void* snmalloc_pagemap_global_get(snmalloc::PagemapConfig const**);
-
-  /**
-   * Mixin used by `ChunkMap` to access the global pagemap via a
-   * type-checked C interface.  This should be used when another library (e.g.
-   * your C standard library) uses snmalloc and you wish to use a different
-   * configuration in your program or library, but wish to share a pagemap so
-   * that either version can deallocate memory.
-   */
-  class ExternalGlobalPagemap
-  {
-    /**
-     * A pointer to the pagemap.
-     */
-    inline static ChunkmapPagemap* external_pagemap;
-
-  public:
-    /**
-     * Returns the exported pagemap.
-     * Accesses the pagemap via the C ABI accessor and casts it to
-     * the expected type, failing in cases of ABI mismatch.
-     */
-    static ChunkmapPagemap& pagemap()
-    {
-      if (external_pagemap == nullptr)
-      {
-        const snmalloc::PagemapConfig* c = nullptr;
-        void* raw_pagemap = snmalloc_pagemap_global_get(&c);
-        external_pagemap = ChunkmapPagemap::cast_to_pagemap(raw_pagemap, c);
-        if (!external_pagemap)
-        {
-          Pal::error("Incorrect ABI of global pagemap.");
-        }
-      }
-      return *external_pagemap;
-    }
-  };
+  extern "C" void*
+  snmalloc_chunkmap_global_get(snmalloc::PagemapConfig const**);
 
   /**
    * Class that defines an interface to the pagemap.  This is provided to
@@ -165,7 +100,7 @@ namespace snmalloc
    * implementation (for example, to move pagemap updates to a different
    * protection domain).
    */
-  template<typename PagemapProvider = GlobalPagemap>
+  template<typename PagemapProvider = GlobalChunkmap>
   struct DefaultChunkMap
   {
     /**
@@ -179,53 +114,45 @@ namespace snmalloc
       return PagemapProvider::pagemap().get(p);
     }
 
-    /**
-     * Get the pagemap entry corresponding to a specific address.
-     */
-    static uint8_t get(void* p)
-    {
-      return get(address_cast(p));
-    }
-
     /**
      * Set a pagemap entry indicating that there is a superslab at the
      * specified index.
      */
-    static void set_slab(Superslab* slab)
+    static void set_slab(CapPtr<Superslab, CBChunk> slab)
     {
-      set(slab, static_cast<size_t>(CMSuperslab));
+      set(address_cast(slab), static_cast<size_t>(CMSuperslab));
     }
     /**
      * Add a pagemap entry indicating that a medium slab has been allocated.
      */
-    static void set_slab(Mediumslab* slab)
+    static void set_slab(CapPtr<Mediumslab, CBChunk> slab)
     {
-      set(slab, static_cast<size_t>(CMMediumslab));
+      set(address_cast(slab), static_cast<size_t>(CMMediumslab));
     }
     /**
      * Remove an entry from the pagemap corresponding to a superslab.
      */
-    static void clear_slab(Superslab* slab)
+    static void clear_slab(CapPtr<Superslab, CBChunk> slab)
     {
-      SNMALLOC_ASSERT(get(slab) == CMSuperslab);
-      set(slab, static_cast<size_t>(CMNotOurs));
+      SNMALLOC_ASSERT(get(address_cast(slab)) == CMSuperslab);
+      set(address_cast(slab), static_cast<size_t>(CMNotOurs));
     }
     /**
      * Remove an entry corresponding to a medium slab.
      */
-    static void clear_slab(Mediumslab* slab)
+    static void clear_slab(CapPtr<Mediumslab, CBChunk> slab)
     {
-      SNMALLOC_ASSERT(get(slab) == CMMediumslab);
-      set(slab, static_cast<size_t>(CMNotOurs));
+      SNMALLOC_ASSERT(get(address_cast(slab)) == CMMediumslab);
+      set(address_cast(slab), static_cast<size_t>(CMNotOurs));
     }
     /**
      * Update the pagemap to reflect a large allocation, of `size` bytes from
      * address `p`.
      */
-    static void set_large_size(void* p, size_t size)
+    static void set_large_size(CapPtr<Largeslab, CBChunk> p, size_t size)
     {
       size_t size_bits = bits::next_pow2_bits(size);
-      set(p, static_cast<uint8_t>(size_bits));
+      set(address_cast(p), static_cast<uint8_t>(size_bits));
       // Set redirect slide
       auto ss = address_cast(p) + SUPERSLAB_SIZE;
       for (size_t i = 0; i < size_bits - SUPERSLAB_BITS; i++)
@@ -240,7 +167,7 @@ namespace snmalloc
      * Update the pagemap to remove a large allocation, of `size` bytes from
      * address `p`.
      */
-    static void clear_large_size(void* vp, size_t size)
+    static void clear_large_size(CapPtr<Largeslab, CBChunk> vp, size_t size)
     {
       auto p = address_cast(vp);
       size_t rounded_size = bits::next_pow2(size);
@@ -255,9 +182,9 @@ namespace snmalloc
      * interface and exists to make it easy to reuse the code in the public
      * methods in other pagemap adaptors.
      */
-    static void set(void* p, uint8_t x)
+    static void set(address_t p, uint8_t x)
     {
-      PagemapProvider::pagemap().set(address_cast(p), x);
+      PagemapProvider::pagemap().set(p, x);
     }
   };
 
