@@ -152,12 +152,10 @@ namespace snmalloc
       return pointer_align_down<SUPERSLAB_SIZE, Slab>(p.as_void()) == p;
     }
 
-    template<capptr_bounds B>
-    SNMALLOC_FAST_PATH static bool
-    is_start_of_object(CapPtr<Metaslab, B> self, address_t p)
+    SNMALLOC_FAST_PATH bool is_start_of_object(address_t p)
     {
       return is_multiple_of_sizeclass(
-        self->sizeclass(), SLAB_SIZE - (p - address_align_down<SLAB_SIZE>(p)));
+        sizeclass(), SLAB_SIZE - (p - address_align_down<SLAB_SIZE>(p)));
     }
 
     /**
@@ -176,9 +174,9 @@ namespace snmalloc
       SNMALLOC_ASSERT(!self->is_full());
 
       self->free_queue.close(fast_free_list, entropy);
-      auto n = fast_free_list.take(entropy);
-      auto n_slab = Aal::capptr_rebound(self.as_void(), n);
-      auto meta = Metaslab::get_slab(n_slab);
+      auto p = fast_free_list.take(entropy);
+      auto slab = Aal::capptr_rebound(self.as_void(), p);
+      auto meta = Metaslab::get_slab(slab);
 
       entropy.refresh_bits();
 
@@ -186,8 +184,7 @@ namespace snmalloc
       self->remove();
       self->set_full(meta);
 
-      auto p = remove_cache_friendly_offset(n, self->sizeclass());
-      SNMALLOC_ASSERT(is_start_of_object(self, address_cast(p)));
+      SNMALLOC_ASSERT(self->is_start_of_object(address_cast(p)));
 
       self->debug_slab_invariant(meta, entropy);
 
@@ -203,7 +200,8 @@ namespace snmalloc
         UNUSED(rsize);
       }
 
-      return capptr_export(p);
+      // TODO: Should this be zeroing the FreeObject state?
+      return capptr_export(p.as_void());
     }
 
     template<capptr_bounds B>
