@@ -8,20 +8,37 @@
 
 namespace snmalloc
 {
+  /**
+   * Superslabs are, to first approximation, a `CHUNK_SIZE`-sized and -aligned
+   * region of address space, internally composed of a header (a `Superslab`
+   * structure) followed by an array of `Slab`s, each `SLAB_SIZE`-sized and
+   * -aligned.  Each active `Slab` holds an array of identically sized
+   * allocations strung on an invasive free list, which is lazily constructed
+   * from a bump-pointer allocator (see `Metaslab::alloc_new_list`).
+   *
+   * In order to minimize overheads, Slab metadata is held externally, in
+   * `Metaslab` structures; all `Metaslab`s for the Slabs within a Superslab are
+   * densely packed within the `Superslab` structure itself.  Moreover, as the
+   * `Superslab` structure is typically much smaller than `SLAB_SIZE`, a "short
+   * Slab" is overlaid with the `Superslab`.  This short Slab can hold only
+   * allocations that are smaller than the `SLAB_SIZE - sizeof(Superslab)`
+   * bytes; see `Superslab::is_short_sizeclass`.  The Metaslab state for a short
+   * slabs is constructed in a way that avoids branches on fast paths;
+   * effectively, the object slots that overlay the `Superslab` at the start are
+   * omitted from consideration.
+   */
   class Superslab : public Allocslab
   {
-    // This is the view of a 16 mb superslab when it is being used to allocate
-    // 64 kb slabs.
   private:
-    friend DLList<Superslab>;
+    friend DLList<Superslab, CapPtrCBChunk>;
 
     // Keep the allocator pointer on a separate cache line. It is read by
     // other threads, and does not change, so we avoid false sharing.
     alignas(CACHELINE_SIZE)
       // The superslab is kept on a doubly linked list of superslabs which
       // have some space.
-      Superslab* next;
-    Superslab* prev;
+      CapPtr<Superslab, CBChunk> next;
+    CapPtr<Superslab, CBChunk> prev;
 
     // This is a reference to the first unused slab in the free slab list
     // It is does not contain the short slab, which is handled using a bit
@@ -42,9 +59,10 @@ namespace snmalloc
     ModArray<SLAB_COUNT, Metaslab> meta;
 
     // Used size_t as results in better code in MSVC
-    size_t slab_to_index(Slab* slab)
+    template<capptr_bounds B>
+    size_t slab_to_index(CapPtr<Slab, B> slab)
     {
-      auto res = (pointer_diff(this, slab) >> SLAB_BITS);
+      auto res = (pointer_diff(this, slab.unsafe_capptr) >> SLAB_BITS);
       SNMALLOC_ASSERT(res == static_cast<uint8_t>(res));
       return static_cast<uint8_t>(res);
     }
@@ -65,16 +83,39 @@ namespace snmalloc
       StatusChange = 2
     };
 
-    static Superslab* get(const void* p)
+    /**
+     * Given a highly-privileged pointer pointing to or within an object in
+     * this slab, return a pointer to the slab headers.
+     *
+     * In debug builds on StrictProvenance architectures, we will enforce the
+     * slab bounds on this returned pointer.  In non-debug builds, we will
+     * return a highly-privileged pointer (i.e., CBArena) instead as these
+     * pointers are not exposed from the allocator.
+     */
+    template<typename T, capptr_bounds B>
+    static SNMALLOC_FAST_PATH CapPtr<Superslab, capptr_bound_chunkd_bounds<B>()>
+    get(CapPtr<T, B> p)
     {
-      return pointer_align_down<SUPERSLAB_SIZE, Superslab>(
-        const_cast<void*>(p));
+      static_assert(B == CBArena || B == CBChunkD || B == CBChunk);
+
+      return capptr_bound_chunkd(
+        pointer_align_down<SUPERSLAB_SIZE, Superslab>(p.as_void()),
+        SUPERSLAB_SIZE);
     }
 
     static bool is_short_sizeclass(sizeclass_t sizeclass)
     {
-      constexpr sizeclass_t h = size_to_sizeclass_const(sizeof(Superslab));
-      return sizeclass <= h;
+      static_assert(SLAB_SIZE > sizeof(Superslab), "Meta data requires this.");
+      /*
+       * size_to_sizeclass_const rounds *up* and returns the smallest class that
+       * could contain (and so may be larger than) the free space available for
+       * the short slab.  While we could detect the exact fit case and compare
+       * `<= h` therein, it's simpler to just treat this class as a strict upper
+       * bound and only permit strictly smaller classes in short slabs.
+       */
+      constexpr sizeclass_t h =
+        size_to_sizeclass_const(SLAB_SIZE - sizeof(Superslab));
+      return sizeclass < h;
     }
 
     void init(RemoteAllocator* alloc)
@@ -107,7 +148,7 @@ namespace snmalloc
       auto curr = head;
       for (size_t i = 0; i < SLAB_COUNT - used - 1; i++)
       {
-        curr = (curr + meta[curr].next + 1) & (SLAB_COUNT - 1);
+        curr = (curr + meta[curr].next() + 1) & (SLAB_COUNT - 1);
       }
       if (curr != 0)
         abort();
@@ -154,69 +195,56 @@ namespace snmalloc
       return Full;
     }
 
-    Metaslab& get_meta(Slab* slab)
+    template<capptr_bounds B>
+    CapPtr<Metaslab, B> get_meta(CapPtr<Slab, B> slab)
     {
-      return meta[slab_to_index(slab)];
+      return CapPtr<Metaslab, B>(&meta[slab_to_index(slab)]);
     }
 
-    Slab* alloc_short_slab(sizeclass_t sizeclass)
+    static CapPtr<Slab, CBChunk>
+    alloc_short_slab(CapPtr<Superslab, CBChunk> self, sizeclass_t sizeclass)
     {
-      if ((used & 1) == 1)
-        return alloc_slab(sizeclass);
+      if ((self->used & 1) == 1)
+        return alloc_slab(self, sizeclass);
 
-      meta[0].head = nullptr;
-      // Set up meta data as if the entire slab has been turned into a free
-      // list. This means we don't have to check for special cases where we have
-      // returned all the elements, but this is a slab that is still being bump
-      // allocated from. Hence, the bump allocator slab will never be returned
-      // for use in another size class.
-      meta[0].allocated = static_cast<uint16_t>(
-        (SLAB_SIZE - get_initial_offset(sizeclass, true)) /
-        sizeclass_to_size(sizeclass));
-      meta[0].link = 1;
-      meta[0].needed = 1;
-      meta[0].sizeclass = static_cast<uint8_t>(sizeclass);
+      auto slab = self.template as_reinterpret<Slab>();
+      auto& metaz = self->meta[0];
 
-      used++;
-      return reinterpret_cast<Slab*>(this);
+      metaz.initialise(sizeclass, slab);
+
+      self->used++;
+      return slab;
     }
 
-    Slab* alloc_slab(sizeclass_t sizeclass)
+    static CapPtr<Slab, CBChunk>
+    alloc_slab(CapPtr<Superslab, CBChunk> self, sizeclass_t sizeclass)
     {
-      uint8_t h = head;
-      Slab* slab = pointer_offset(
-        reinterpret_cast<Slab*>(this), (static_cast<size_t>(h) << SLAB_BITS));
+      uint8_t h = self->head;
+      auto slab = pointer_offset(self, (static_cast<size_t>(h) << SLAB_BITS))
+                    .template as_static<Slab>();
 
-      uint8_t n = meta[h].next;
+      auto& metah = self->meta[h];
+      uint8_t n = metah.next();
 
-      meta[h].head = nullptr;
-      // Set up meta data as if the entire slab has been turned into a free
-      // list. This means we don't have to check for special cases where we have
-      // returned all the elements, but this is a slab that is still being bump
-      // allocated from. Hence, the bump allocator slab will never be returned
-      // for use in another size class.
-      meta[h].allocated = static_cast<uint16_t>(
-        (SLAB_SIZE - get_initial_offset(sizeclass, false)) /
-        sizeclass_to_size(sizeclass));
-      meta[h].needed = 1;
-      meta[h].link = 1;
-      meta[h].sizeclass = static_cast<uint8_t>(sizeclass);
+      metah.initialise(sizeclass, slab);
 
-      head = h + n + 1;
-      used += 2;
+      self->head = h + n + 1;
+      self->used += 2;
 
       return slab;
     }
 
     // Returns true, if this alters the value of get_status
-    Action dealloc_slab(Slab* slab)
+    template<capptr_bounds B>
+    Action dealloc_slab(CapPtr<Slab, B> slab)
     {
+      static_assert(B == CBArena || B == CBChunkD || B == CBChunk);
+
       // This is not the short slab.
       uint8_t index = static_cast<uint8_t>(slab_to_index(slab));
       uint8_t n = head - index - 1;
 
-      meta[index].sizeclass = 0;
-      meta[index].next = n;
+      meta[index].next() = n;
       head = index;
       bool was_almost_full = is_almost_full();
       used -= 2;
