@@ -62,7 +62,7 @@ namespace snmalloc
     template<bool committed>
     static void* reserve_aligned(size_t size) noexcept
     {
-      SNMALLOC_ASSERT(size == bits::next_pow2(size));
+      SNMALLOC_ASSERT(bits::is_pow2(size));
       SNMALLOC_ASSERT(size >= minimum_alloc_size);
       size_t align = size;
 
