@@ -1,22 +1,13 @@
 #pragma once
 
 #if defined(__HAIKU__)
-#  include <cassert>
-
-namespace snmalloc
-{
-  template<typename... Args>
-  int getentropy(Args...)
-  {
-    assert(0 && "Unreachable path");
-    return -1;
-  }
-}
 
 #  include "pal_posix.h"
 
 #  include <sys/mman.h>
 
+extern uint64_t secure_get_random();
+
 namespace snmalloc
 {
   /**
@@ -30,11 +21,8 @@ namespace snmalloc
      * Bitmap of PalFeatures flags indicating the optional features that this
      * PAL supports.
      *
-     * Disable the POSIX default implementation of Entropy as not supported on
-     * Haiku.
      */
-    static constexpr uint64_t pal_features =
-      PALPOSIX::pal_features & ~(Entropy);
+    static constexpr uint64_t pal_features = PALPOSIX::pal_features;
 
     /**
      * Haiku requires an explicit no-reserve flag in `mmap` to guarantee lazy
@@ -49,7 +37,18 @@ namespace snmalloc
     static void notify_not_using(void* p, size_t size) noexcept
     {
       SNMALLOC_ASSERT(is_aligned_block<page_size>(p, size));
-      posix_madvise(p, size, POSIX_MADV_DONTNEED);
+#  ifdef USE_POSIX_COMMIT_CHECKS
+      memset(p, 0x5a, size);
+#  endif
+      madvise(p, size, MADV_FREE);
+#  ifdef USE_POSIX_COMMIT_CHECKS
+      mprotect(p, size, PROT_NONE);
+#  endif
+    }
+
+    static uint64_t get_entropy64()
+    {
+      return secure_get_random();
     }
   };
 } // namespace snmalloc
