@@ -117,9 +117,14 @@ namespace snmalloc
      * Bitmap of PalFeatures flags indicating the optional features that this
      * PAL supports.
      *
-     * POSIX systems are assumed to support lazy commit.
+     * POSIX systems are assumed to support lazy commit. The build system checks
+     * getentropy is available, only then this PAL supports Entropy.
      */
-    static constexpr uint64_t pal_features = LazyCommit | Entropy;
+    static constexpr uint64_t pal_features = LazyCommit
+#if defined(SNMALLOC_PLATFORM_HAS_GETENTROPY)
+      | Entropy
+#endif
+      ;
 
     static constexpr size_t page_size = Aal::smallest_page_size;
 
@@ -290,11 +295,14 @@ namespace snmalloc
       }
       else
       {
+#ifdef SNMALLOC_PLATFORM_HAS_GETENTROPY
         uint64_t result;
         if (getentropy(&result, sizeof(result)) != 0)
           error("Failed to get system randomness");
         return result;
+#endif
       }
+      error("Entropy requested on platform that does not provide entropy");
     }
   };
 } // namespace snmalloc
